# 导出Blender集合

import bpy

extension_is_required = False
glTF_extension_name = "DS_collections"

class CollectionsProperties(bpy.types.PropertyGroup):
    enabled: bpy.props.BoolProperty(
        name = '集合结构',
        description = '导出Blender中的集合(Collection)',
        default = False
    )


def register():
    bpy.utils.register_class(CollectionsProperties)
    bpy.types.Scene.CollectionsProperties = bpy.props.PointerProperty(type = CollectionsProperties)


def unregister():
    bpy.utils.unregister_class(CollectionsProperties)
    del bpy.types.Scene.CollectionsProperties


def pannel_draw(panel, context):
    layout = panel.layout
    layout.prop(bpy.context.scene.CollectionsProperties, 'enabled')



def user_extension_init(user_extension):
    user_extension.objectName2CollectionIdx = {}

    user_extension.collections = []
    def traverse(collection, parentCollectionIdx):
        collectionIdx = parentCollectionIdx + 1
        user_extension.collections.insert(collectionIdx, collection.name)

        for object in collection.objects:
            user_extension.objectName2CollectionIdx[object.name] = collectionIdx

        beforeCount = 0
        childCollections = []
        for childCollection in collection.children:
            child = traverse(childCollection, collectionIdx + beforeCount)
            childCollections.append(child['tree'])
            beforeCount = beforeCount + child['beforeCount']
        if len(childCollections) == 0:
            return {
                'tree': collectionIdx,
                'beforeCount': 1
            }
        else:
            childTree = {}
            childTree[collectionIdx] = childCollections
            return {
                'tree': childTree,
                'beforeCount': 1 + beforeCount
            }
    user_extension.collectionTree = traverse(bpy.data.scenes[0].collection, -1)['tree']

def gather_node_hook(user_extension, gltf2_node, blender_object, export_settings):
    if not bpy.context.scene.CollectionsProperties.enabled: return

    gltf2_node.extensions[glTF_extension_name] = user_extension.Extension(
        name = glTF_extension_name,
        extension = {
            'collection': user_extension.objectName2CollectionIdx[blender_object.name]
        },
        required = extension_is_required
    )

def gather_gltf_hook(user_extension, gltf2_plan, export_settings):
    if not bpy.context.scene.CollectionsProperties.enabled: return

    gltf2_plan.extensions[glTF_extension_name] = user_extension.Extension(
        name = glTF_extension_name,
        extension = {
            'collections': user_extension.collections,
            'collectionTree': user_extension.collectionTree
        },
        required = extension_is_required
    )